@@ -1,8 +1,10 @@
 /*
- * i-scream libstatgrab
- * http://www.i-scream.org
- * Copyright (C) 2000-2013 i-scream
- * Copyright (C) 2010-2013 Jens Rehsack
+ * libstatgrab
+ * https://libstatgrab.org
+ * Copyright (C) 2003-2004 Peter Saunders
+ * Copyright (C) 2003-2019 Tim Bishop
+ * Copyright (C) 2003-2013 Adam Sampson
+ * Copyright (C) 2012-2019 Jens Rehsack
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,11 +31,26 @@
 
 #include "helpers.h"
 
+#ifdef HAVE_LOG4CPLUS_INITIALIZE
+static void *l4cplus_initializer;
+
+static void
+cleanup_logging(void)
+{
+	log4cplus_deinitialize(l4cplus_initializer);
+}
+#endif
+
 int main(int argc, char **argv){
 
 	sg_network_iface_stats *network_iface_stats;
 	size_t iface_count, i;
 
+#ifdef HAVE_LOG4CPLUS_INITIALIZE
+	l4cplus_initializer = log4cplus_initialize();
+	atexit((void (*)(void))cleanup_logging);
+#endif
+
 	/* Initialise helper - e.g. logging, if any */
 	sg_log_init("libstatgrab-examples", "SGEXAMPLES_LOG_PROPERTIES", argc ? argv[0] : NULL);
 
