@@ -1,8 +1,10 @@
 /*
- * i-scream libstatgrab
- * http://www.i-scream.org
- * Copyright (C) 2000-2013 i-scream
- * Copyright (C) 2010-2013 Jens Rehsack
+ * libstatgrab
+ * https://libstatgrab.org
+ * Copyright (C) 2003-2004 Peter Saunders
+ * Copyright (C) 2003-2019 Tim Bishop
+ * Copyright (C) 2003-2013 Adam Sampson
+ * Copyright (C) 2012-2019 Jens Rehsack
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,9 +18,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
- *
- * $Id$
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #include <stdio.h>
@@ -30,6 +31,16 @@
 
 static int quit;
 
+#ifdef HAVE_LOG4CPLUS_INITIALIZE
+static void *l4cplus_initializer;
+
+static void
+cleanup_logging(void)
+{
+	log4cplus_deinitialize(l4cplus_initializer);
+}
+#endif
+
 int main(int argc, char **argv){
 
 	extern char *optarg;
@@ -38,6 +49,11 @@ int main(int argc, char **argv){
 	int delay = 1;
 	sg_process_count *process_stat;
 
+#ifdef HAVE_LOG4CPLUS_INITIALIZE
+	l4cplus_initializer = log4cplus_initialize();
+	atexit((void (*)(void))cleanup_logging);
+#endif
+
 	while ((c = getopt(argc, argv, "d:")) != -1){
 		switch (c){
 			case 'd':
