@@ -1,8 +1,10 @@
 /*
- * i-scream libstatgrab
- * http://www.i-scream.org
- * Copyright (C) 2000-2013 i-scream
- * Copyright (C) 2010-2013 Jens Rehsack
+ * libstatgrab
+ * https://libstatgrab.org
+ * Copyright (C) 2003-2004 Peter Saunders
+ * Copyright (C) 2003-2019 Tim Bishop
+ * Copyright (C) 2003-2013 Adam Sampson
+ * Copyright (C) 2012-2019 Jens Rehsack
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,9 +18,8 @@
  * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
- *
- * $Id$
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #include "tools.h"
@@ -219,6 +220,7 @@ sg_get_cpu_stats_int(sg_cpu_stats *cpu_stats_buf) {
 				&cpu_stats_buf->iowait);
 
 			if (proc_stat_cpu < 4 || proc_stat_cpu > 5) {
+				fclose(f);
 				RETURN_WITH_SET_ERROR("cpu", SG_ERROR_PARSE, "cpu");
 			}
 
@@ -228,6 +230,7 @@ sg_get_cpu_stats_int(sg_cpu_stats *cpu_stats_buf) {
 			proc_stat_cpu = sscanf(line, "intr %llu", &cpu_stats_buf->interrupts);
 
 			if (proc_stat_cpu != 1) {
+				fclose(f);
 				RETURN_WITH_SET_ERROR("cpu", SG_ERROR_PARSE, "intr");
 			}
 
@@ -237,6 +240,7 @@ sg_get_cpu_stats_int(sg_cpu_stats *cpu_stats_buf) {
 			proc_stat_cpu = sscanf(line, "ctxt %llu", &cpu_stats_buf->context_switches);
 
 			if (proc_stat_cpu != 1) {
+				fclose(f);
 				RETURN_WITH_SET_ERROR("cpu", SG_ERROR_PARSE, "ctxt");
 			}
 
@@ -246,6 +250,7 @@ sg_get_cpu_stats_int(sg_cpu_stats *cpu_stats_buf) {
 			proc_stat_cpu = sscanf(line, "softirq %llu", &cpu_stats_buf->soft_interrupts);
 
 			if (proc_stat_cpu != 1) {
+				fclose(f);
 				RETURN_WITH_SET_ERROR("cpu", SG_ERROR_PARSE, "softirq");
 			}
 
@@ -255,6 +260,10 @@ sg_get_cpu_stats_int(sg_cpu_stats *cpu_stats_buf) {
 
 	fclose(f);
 
+	if( matched < 4 ) {
+		RETURN_WITH_SET_ERROR("cpu", SG_ERROR_PARSE, "not all from 'cpu', 'intr', 'ctxt', 'softirq' found in '/proc/stat'");
+	}
+
 	cpu_stats_buf->total = cpu_stats_buf->user + cpu_stats_buf->nice + cpu_stats_buf->kernel + cpu_stats_buf->idle;
 #elif defined(ALLBSD)
 # if defined(HAVE_HOST_STATISTICS) || defined(HAVE_HOST_STATISTICS64)
@@ -434,7 +443,7 @@ VECTOR_INIT_INFO_EMPTY_INIT(sg_cpu_percents);
 
 static sg_error
 sg_get_cpu_percents_int(sg_cpu_percents *cpu_percent_buf, const sg_cpu_stats *cpu_stats_buf){
-
+#ifndef WIN32
 	cpu_percent_buf->user =  ((double)cpu_stats_buf->user) / ((double)cpu_stats_buf->total) * 100;
 	cpu_percent_buf->kernel =  ((double)cpu_stats_buf->kernel) / ((double)cpu_stats_buf->total) * 100;
 	cpu_percent_buf->idle = ((double)cpu_stats_buf->idle) / ((double)cpu_stats_buf->total) * 100;
@@ -442,52 +451,45 @@ sg_get_cpu_percents_int(sg_cpu_percents *cpu_percent_buf, const sg_cpu_stats *cp
 	cpu_percent_buf->swap = ((double)cpu_stats_buf->swap) / ((double)cpu_stats_buf->total) * 100;
 	cpu_percent_buf->nice = ((double)cpu_stats_buf->nice) / ((double)cpu_stats_buf->total) * 100;
 	cpu_percent_buf->time_taken = cpu_stats_buf->systime;
+#else
+
+	/* this is nonsense - according to the documentation at
+	 * http://support.microsoft.com/kb/262938 is it possible for other
+	 * applications to read the same global counter and reset the the
+	 * measure points.
+	 *
+	 * better take a look to
+	 * - http://technet.microsoft.com/en-us/library/cc737309%28WS.10%29.aspx
+	 * - SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION at
+	 *   http://msdn.microsoft.com/en-us/library/ms724509%28VS.85%29.aspx
+	 * It's unlikely that this API will be removed before full replacement
+	 * API is available.
+	 */
 
-	return SG_ERROR_NONE;
-}
-#if 0
-/* this is nonsense - according to the documentation at
- * http://support.microsoft.com/kb/262938 is it possible for other
- * applications to read the same global counter and reset the the
- * measure points.
- *
- * better take a look to
- * - http://technet.microsoft.com/en-us/library/cc737309%28WS.10%29.aspx
- * - SYSTEM_PROCESSOR_PERFORMANCE_INFORMATION at
- *   http://msdn.microsoft.com/en-us/library/ms724509%28VS.85%29.aspx
- * It's unlikely that this API will be removed before full replacement
- * API is available.
- */
-#ifdef WIN32
 	double result;
 
 	if(read_counter_double(SG_WIN32_PROC_USER, &result)) {
-		sg_set_error(SG_ERROR_PDHREAD, PDH_USER);
-		return NULL;
+		RETURN_WITH_SET_ERROR_WITH_ERRNO("cpu", SG_ERROR_KSTAT_READ, "SG_WIN32_PROC_USER");
 	}
-	cpu_usage->user = (float)result;
+	cpu_percent_buf->user = (float)result;
 	if(read_counter_double(SG_WIN32_PROC_PRIV, &result)) {
-		sg_set_error(SG_ERROR_PDHREAD, PDH_PRIV);
-		return NULL;
+		RETURN_WITH_SET_ERROR_WITH_ERRNO("cpu", SG_ERROR_KSTAT_READ, "SG_WIN32_PROC_PRIV");
 	}
-	cpu_usage->kernel = (float)result;
+	cpu_percent_buf->kernel = (float)result;
 	if(read_counter_double(SG_WIN32_PROC_IDLE, &result)) {
-		sg_set_error(SG_ERROR_PDHREAD, PDH_IDLE);
-		return NULL;
+		RETURN_WITH_SET_ERROR_WITH_ERRNO("cpu", SG_ERROR_KSTAT_READ, "SG_WIN32_PROC_IDLE");
 	}
 	/* win2000 does not have an idle counter, but does have %activity
 	 * so convert it to idle */
-	cpu_usage->idle = 100 - (float)result;
+	cpu_percent_buf->idle = 100 - (float)result;
 	if(read_counter_double(SG_WIN32_PROC_INT, &result)) {
-		sg_set_error(SG_ERROR_PDHREAD, PDH_INTER);
-		return NULL;
+		RETURN_WITH_SET_ERROR_WITH_ERRNO("cpu", SG_ERROR_KSTAT_READ, "SG_WIN32_PROC_INT");
 	}
-	cpu_usage->iowait = (float)result;
+	cpu_percent_buf->iowait = (float)result;
+#endif
 
 	return SG_ERROR_NONE;
-#endif
-#endif
-
+}
 sg_cpu_percents *
 sg_get_cpu_percents_of(sg_cpu_percent_source cps, size_t *entries) {
 
