@@ -1,8 +1,10 @@
 /*
- * i-scream libstatgrab
- * http://www.i-scream.org
- * Copyright (C) 2000-2013 i-scream
- * Copyright (C) 2010-2013 Jens Rehsack
+ * libstatgrab
+ * https://libstatgrab.org
+ * Copyright (C) 2003-2004 Peter Saunders
+ * Copyright (C) 2003-2019 Tim Bishop
+ * Copyright (C) 2003-2013 Adam Sampson
+ * Copyright (C) 2012-2019 Jens Rehsack
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -16,10 +18,8 @@
  * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307 USA
- *
- * $Id$
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #define __NEED_SG_GET_SYS_PAGE_SIZE
@@ -179,7 +179,7 @@ free_pids_in_proc_dir( struct pids_in_proc_dir_t *pipd, bool include_children )
 	return next;
 }
 
-#define PIPD_IS_FULL(pipd,size) (((size) - ((offsetof(struct pids_in_proc_dir_t, items)))/sizeof(pid_t)) < ((pipd)->nitems + 1))
+#define PIPD_IS_FULL(pipd,size) (offsetof(struct pids_in_proc_dir_t, items[(pipd)->nitems+1]) > (size_t)(size))
 
 static struct pids_in_proc_dir_t *
 add_pid_to_pids_in_proc_dir( pid_t pid, struct pids_in_proc_dir_t *pipd ) {
@@ -293,7 +293,6 @@ adjust_procname_cmndline(char *proctitle, size_t len) {
 }
 #endif
 
-
 static sg_error
 sg_get_process_stats_int(sg_vector **proc_stats_vector_ptr) {
 
@@ -302,12 +301,16 @@ sg_get_process_stats_int(sg_vector **proc_stats_vector_ptr) {
 	time_t now = time(NULL);
 
 #if defined(LINUX)
+#define READBUF_LENGTH 4095
+	static const char proc_stat_fmt_base[] = "%" FMT_PID_T " %s %%c %" FMT_PID_T " %" FMT_PID_T " %%*d %%*d %%*d %%*u %%*u %%*u %%*u %%*u %%lu %%lu %%*d %%*d %%*d %%d %%*d %%*d %%llu %%llu %%llu %%*u %%*u %%*u %%*u %%*u %%*u %%*u %%*u %%*u %%*u %%*u %%*u %%*u %%*d %%*d";
+	static const char proc_stat_fmt_default[] = FMT_PID_T " %4095s %c " FMT_PID_T " " FMT_PID_T " %*d %*d %*d %*u %*u %*u %*u %*u %lu %lu %*d %*d %*d %d %*d %*d %llu %llu %llu %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*d %*d";
 	struct pids_in_proc_dir_t *pids_in_proc_dir;
 	size_t pid_item = 0;
 	char filename[MAX_FILE_LENGTH];
 	FILE *f;
 	char s;
-	char read_buf[4096];
+	char read_buf[READBUF_LENGTH+1];
+	char proc_stat_fmt[sizeof proc_stat_fmt_base];
 	char *read_ptr;
 	/* XXX or we detect max command line length in ./configure */
 	unsigned long stime, utime;
@@ -466,6 +469,43 @@ sg_get_process_stats_int(sg_vector **proc_stats_vector_ptr) {
 			continue;
 		}
 
+		snprintf(filename, MAX_FILE_LENGTH, PROC_LOCATION "/" FMT_PID_T "/comm", pids_in_proc_dir->items[pid_item]);
+		if( ( f = fopen(filename, "r") ) == NULL ) {
+			/* Open failed.. */
+			proc_stat_fmt[0] = read_buf[0] = '\0';
+		}
+		else {
+			char *tok, *tok_e;
+			char comm_jmp[32] = { '\0' }; /* procfs documents comm with 16 chars max */
+			if(NULL == fgets(read_buf, READBUF_LENGTH, f)) {
+				/* Read error or EOF before any char read -> finally no desired information */
+				fclose(f);
+				continue;
+			}
+			fclose(f);
+			read_buf[sizeof read_buf - 1] = '\0';
+
+			tok = read_buf;
+			tok_e = read_buf + strlen(read_buf) - 1;
+			if(*tok_e == '\n')
+				*tok_e-- = '\0';
+
+			strncat(comm_jmp, "%*s", sizeof(comm_jmp) - strlen(comm_jmp) - 1);
+			while(tok <= tok_e) {
+				if(isspace(*tok)) {
+					strncat(comm_jmp, " ", sizeof(comm_jmp) - strlen(comm_jmp) - 1);
+					strncat(comm_jmp, "%*s", sizeof(comm_jmp) - strlen(comm_jmp) - 1);
+
+					while(isspace(*tok) && tok <= tok_e)
+						++tok;
+				}
+
+				++tok;
+			}
+
+			snprintf(proc_stat_fmt, sizeof(proc_stat_fmt), proc_stat_fmt_base, comm_jmp);
+		}
+
 		snprintf(filename, MAX_FILE_LENGTH, PROC_LOCATION "/" FMT_PID_T "/stat", pids_in_proc_dir->items[pid_item]);
 		if( ( f = fopen(filename, "r") ) == NULL ) {
 			/* Open failed.. Process since vanished, or the path was too long.
@@ -476,10 +516,26 @@ sg_get_process_stats_int(sg_vector **proc_stats_vector_ptr) {
 
 		VECTOR_UPDATE(proc_stats_vector_ptr, proc_items + 1, proc_stats_ptr, sg_process_stats);
 
-		fscanf(f, FMT_PID_T "%4095s %c " FMT_PID_T " " FMT_PID_T " %*d %*d %*d %*u %*u %*u %*u %*u %lu %lu %*d %*d %*d %d %*d %*d %llu %llu %llu %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*u %*d %*d\n",
-			   &proc_stats_ptr[proc_items].pid, read_buf, &s, &proc_stats_ptr[proc_items].parent,
-			   &proc_stats_ptr[proc_items].pgid, &utime, &stime, &proc_stats_ptr[proc_items].nice,
-			   &starttime, &proc_stats_ptr[proc_items].proc_size, &proc_stats_ptr[proc_items].proc_resident);
+		if( proc_stat_fmt[0] ) {
+			rc = fscanf(f, proc_stat_fmt,
+			       &proc_stats_ptr[proc_items].pid, &s, &proc_stats_ptr[proc_items].parent,
+			       &proc_stats_ptr[proc_items].pgid, &utime, &stime, &proc_stats_ptr[proc_items].nice,
+			       &starttime, &proc_stats_ptr[proc_items].proc_size, &proc_stats_ptr[proc_items].proc_resident);
+		}
+		else {
+			rc = fscanf(f, proc_stat_fmt_default,
+			       &proc_stats_ptr[proc_items].pid, read_buf, &s, &proc_stats_ptr[proc_items].parent,
+			       &proc_stats_ptr[proc_items].pgid, &utime, &stime, &proc_stats_ptr[proc_items].nice,
+			       &starttime, &proc_stats_ptr[proc_items].proc_size, &proc_stats_ptr[proc_items].proc_resident);
+		}
+
+		if( (proc_stat_fmt[0] && rc < 10) || (proc_stat_fmt[0] == '\0' && rc < 11)) {
+			/* Read failed.. Process vanished?
+			 * Ah well, move onwards to the next one */
+			++pid_item;
+			fclose(f);
+			continue;
+		}
 		/* +3 because man page says "Resident  Set Size: number of pages the process has in real memory, minus 3 for administrative purposes." */
 		proc_stats_ptr[proc_items].proc_resident = (proc_stats_ptr[proc_items].proc_resident + 3) * sys_page_size;
 		switch (s) {
@@ -499,12 +555,14 @@ sg_get_process_stats_int(sg_vector **proc_stats_vector_ptr) {
 		}
 
 		/* pa_name[0] should = '(' */
+		rc = 0;
 		read_ptr = strchr(&read_buf[1], ')');
 		if( read_ptr != NULL ) {
 			*read_ptr = '\0';
+			rc = 1;
 		}
 
-		if( SG_ERROR_NONE != sg_update_string( &proc_stats_ptr[proc_items].process_name, &read_buf[1] )) {
+		if( SG_ERROR_NONE != sg_update_string( &proc_stats_ptr[proc_items].process_name, &read_buf[rc] )) {
 			VECTOR_UPDATE_ERROR_CLEANUP
 			RETURN_FROM_PREVIOUS_ERROR( "process", sg_get_error() );
 		}
@@ -764,8 +822,7 @@ argv_again:
 					goto argv_again;
 				}
 				else {
-					long failing_pid = kp_stats[i].p_pid;
-					INFO_LOG_FMT("process", "sysctl(CTL_KERN.KERN_PROC_ARGS.KERN_PROC_ARGV) failed for pid=%ld", failing_pid);
+					INFO_LOG_FMT("process", "sysctl(CTL_KERN.KERN_PROC_ARGS.KERN_PROC_ARGV) failed for pid=" FMT_PID_T, kp_stats[i].p_pid);
 					continue;
 				}
 			}
@@ -824,6 +881,9 @@ again:
 	size = 0;
 	mib[3] = mib[5] = 0;
 	if( -1 == (rc = sysctl(mib, (unsigned)i, NULL, &size, NULL, 0)) ) {
+		if(kp_stats != NULL)
+			free(kp_stats);
+		free(proctitle);
 		RETURN_WITH_SET_ERROR_WITH_ERRNO("process", SG_ERROR_SYSCTL, "CTL_KERN.KERN_PROC.KERN_PROC_ALL");
 	}
 	if( NULL == (tmp = sg_realloc(kp_stats, size)) ) {
@@ -838,6 +898,8 @@ again:
 		if( errno == ENOMEM ) {
 			goto again;
 		}
+		free(kp_stats);
+		free(proctitle);
 		RETURN_WITH_SET_ERROR_WITH_ERRNO("process", SG_ERROR_SYSCTL, "CTL_KERN.KERN_PROC.KERN_PROC_ALL");
 	}
 
@@ -848,6 +910,15 @@ again:
 	VECTOR_UPDATE(proc_stats_vector_ptr, nprocs, proc_stats_ptr, sg_process_stats);
 	for( i = 0; i < nprocs; ++i ) {
 		const char *name;
+# ifdef HAVE_PROC_PIDINFO
+		ssize_t buf_fill;
+#  ifdef HAVE_STRUCT_PROC_TASKINFO
+		struct proc_taskinfo task_info;
+#  endif
+#  ifdef HAVE_STRUCT_PROC_BSDINFO
+		struct proc_bsdinfo bsd_info;
+#  endif
+# endif
 
 # if defined(HAVE_KINFO_PROC_KI_STAT)
 		if( kp_stats[i].ki_stat == 0 )
@@ -869,6 +940,20 @@ again:
 			continue;
 		}
 
+
+# if defined(HAVE_STRUCT_PROC_TASKINFO) && defined(HAVE_PROC_PIDINFO)
+		buf_fill = proc_pidinfo(kp_stats[i].kp_proc.p_pid, PROC_PIDTASKINFO, 0,  &task_info, sizeof(task_info));
+		if( sizeof(task_info) != buf_fill) {
+			bzero(&task_info, sizeof(task_info));
+		}
+# endif
+# if defined(HAVE_STRUCT_PROC_TASKINFO) && defined(HAVE_PROC_PIDINFO)
+		buf_fill = proc_pidinfo(kp_stats[i].kp_proc.p_pid, PROC_PIDTBSDINFO, 0,  &bsd_info, sizeof(bsd_info));
+		if( sizeof(bsd_info) != buf_fill) {
+			bzero(&bsd_info, sizeof(bsd_info));
+		}
+# endif
+
 # if defined(HAVE_KINFO_PROC_KI_STAT)
 		name = kp_stats[proc_items].ki_comm;
 # elif defined(HAVE_KINFO_PROC_KP_PID)
@@ -913,9 +998,6 @@ again:
 		proc_stats_ptr[proc_items].euid = kp_stats[i].kp_eproc.e_pcred.p_svuid;
 		proc_stats_ptr[proc_items].gid = kp_stats[i].kp_eproc.e_pcred.p_rgid;
 		proc_stats_ptr[proc_items].egid = kp_stats[i].kp_eproc.e_pcred.p_svgid;
-
-		proc_stats_ptr[proc_items].cpu_percent =
-			((double)kp_stats[i].kp_proc.p_pctcpu / FSCALE) * 100.0;
 #  endif
 # elif defined(HAVE_KINFO_PROC_KP_PID)
 		proc_stats_ptr[proc_items].pid = kp_stats[i].kp_pid;
@@ -927,7 +1009,7 @@ again:
 		proc_stats_ptr[proc_items].euid = kp_stats[i].kp_uid;
 		proc_stats_ptr[proc_items].gid = kp_stats[i].kp_rgid;
 		proc_stats_ptr[proc_items].egid = kp_stats[i].kp_svgid;
-#elif defined(HAVE_KINFO_PROC_P_PID)
+# elif defined(HAVE_KINFO_PROC_P_PID)
 		proc_stats_ptr[proc_items].pid = kp_stats[i].p_pid;
 		proc_stats_ptr[proc_items].parent = kp_stats[i].p_ppid == ((pid_t)-1) ? 0 : kp_stats[i].p_ppid;
 		proc_stats_ptr[proc_items].pgid = kp_stats[i].p__pgid;
@@ -942,11 +1024,6 @@ again:
 		proc_stats_ptr[proc_items].parent = kp_stats[i].kp_eproc.e_ppid;
 		proc_stats_ptr[proc_items].pgid = kp_stats[i].kp_eproc.e_pgid;
 		proc_stats_ptr[proc_items].sessid = 0;
-
-		proc_stats_ptr[proc_items].time_spent =
-			kp_stats[i].kp_proc.p_runtime / 1000000;
-		proc_stats_ptr[proc_items].cpu_percent =
-			((double)kp_stats[i].kp_proc.p_pctcpu / FSCALE) * 100.0;
 # endif
 
 
@@ -955,6 +1032,15 @@ again:
 		proc_stats_ptr[proc_items].involuntary_context_switches = 0;
 		proc_stats_ptr[proc_items].start_time = 0;
 
+# ifdef HAVE_STRUCT_PROC_TASKINFO
+		proc_stats_ptr[proc_items].time_spent = (task_info.pti_total_user + task_info.pti_total_system) / 1000000000L;
+		proc_stats_ptr[proc_items].context_switches = task_info.pti_csw;
+# endif
+# ifdef HAVE_STRUCT_PROC_BSDINFO
+		proc_stats_ptr[proc_items].nice = bsd_info.pbi_nice;
+		proc_stats_ptr[proc_items].start_time = (time_t)bsd_info.pbi_start_tvsec;
+# endif
+
 # if defined(HAVE_KINFO_PROC_KI_STAT)
 		/* This is in microseconds */
 		proc_stats_ptr[proc_items].time_spent = kp_stats[i].ki_runtime / 1000000;
@@ -971,6 +1057,14 @@ again:
 			( kp_stats[i].kp_thread.td_uticks +
 			kp_stats[i].kp_thread.td_sticks +
 			kp_stats[i].kp_thread.td_iticks ) / 1000000;
+# elif defined(HAVE_KINFO_PROC_KP_PROC)
+		if(kp_stats[i].kp_proc.p_ru) {
+			proc_stats_ptr[proc_items].voluntary_context_switches = kp_stats[i].kp_proc.p_ru->ru_nvcsw;
+			proc_stats_ptr[proc_items].involuntary_context_switches = kp_stats[i].kp_proc.p_ru->ru_nivcsw;
+		}
+		proc_stats_ptr[proc_items].start_time = kp_stats[i].kp_proc.p_starttime.tv_sec;
+		proc_stats_ptr[proc_items].cpu_percent =
+			((double)kp_stats[i].kp_proc.p_pctcpu / FSCALE) * 100.0;
 # else
 		/* p_cpticks?, p_[usi]ticks? */
 #if 0
@@ -990,6 +1084,9 @@ again:
 # elif defined(HAVE_KINFO_PROC_P_VM_MAP_SIZE)
 		proc_stats_ptr[proc_items].proc_size = kp_stats[i].p_vm_map_size;
 		proc_stats_ptr[proc_items].proc_resident = ((unsigned long long)kp_stats[i].p_vm_rssize) * sys_page_size;
+# elif defined(HAVE_STRUCT_PROC_TASKINFO) && defined(HAVE_PROC_PIDINFO)
+		proc_stats_ptr[proc_items].proc_size = task_info.pti_virtual_size;
+		proc_stats_ptr[proc_items].proc_resident = task_info.pti_resident_size;
 # else
 #if 0
 		/* This is in microseconds */
@@ -997,7 +1094,6 @@ again:
 		/* This is in pages */
 		proc_stats_ptr[proc_items].proc_resident = kp_stats[i].kp_eproc.e_vm.vm_rssize * sys_page_size;
 #endif
-		proc_stats_ptr[proc_items].nice = kp_stats[i].kp_proc.p_nice;
 # endif
 
 # if defined(HAVE_KINFO_PROC_KI_STAT)
@@ -1062,7 +1158,6 @@ again:
 
 # if defined(KERN_PROC_ARGS) || defined(KERN_PROCARGS2)
 		if( 0 != proc_stats_ptr[proc_items].pid ) {
-			char *p;
 			unsigned miblen;
 
 			size = ARG_MAX * sizeof(*proctitle);
@@ -1073,25 +1168,25 @@ again:
 			mib[2] = ((int)proc_stats_ptr[proc_items].pid);
 			mib[3] = KERN_PROC_ARGV;
 			miblen = 4;
-			p = "CTL_KERN.KERN_PROC_ARGS.KERN_PROC_ARGV";
+#define PROC_STATS_SYSCTL_NAME "CTL_KERN.KERN_PROC_ARGS.KERN_PROC_ARGV"
 #  elif defined(KERN_PROC_ARGS) && !defined(KERN_PROC_ARGV)
 			mib[1] = KERN_PROC;
 			mib[2] = KERN_PROC_ARGS;
 			mib[3] = ((int)proc_stats_ptr[proc_items].pid);
 			miblen = 4;
-			p = "CTL_KERN.KERN_PROC.KERN_PROC_ARGS";
+#define PROC_STATS_SYSCTL_NAME "CTL_KERN.KERN_PROC.KERN_PROC_ARGS"
 #  elif defined(KERN_PROCARGS2)
 			mib[1] = KERN_PROCARGS2;
 			mib[2] = ((int)proc_stats_ptr[proc_items].pid);
 			miblen = 3;
-			p = "CTL_KERN.KERN_PROCARGS2";
+#define PROC_STATS_SYSCTL_NAME "CTL_KERN.KERN_PROCARGS2"
 #  endif
 			if( -1 == ( rc = sysctl(mib, miblen, proctitle, &size, NULL, 0) ) ) {
 #  if defined(KERN_PROCARGS2)
 				if( EINVAL == errno )
 					goto print_kernel_proctitle;
 #  endif
-				INFO_LOG_FMT("process", "sysctl(%s) for pid=" FMT_PID_T, p, proc_stats_ptr[proc_items].pid);
+				INFO_LOG_FMT("process", "sysctl(" PROC_STATS_SYSCTL_NAME ") for pid=" FMT_PID_T, proc_stats_ptr[proc_items].pid);
 				continue;
 			}
 
